//-----------------------------------------------------------------
///////////////////////////////////////////////////////////////////
// JavaScript-Bibliothek fr die Objektklasse "Fraction" (Brche)
// Datei:  fraction.js
// Autor:  Alfred Schreiber
// Beginn: 2003-02-01
// Stand:  2003-02-09
///////////////////////////////////////////////////////////////////
//-----------------------------------------------------------------


function Fraction( a, b )
// Definition der Objektklasse Fraction
// Syntax: var f = new Fraction(a,b)
{
  // Eigenschaften 
  // ------------------------------
  this.numerator = a;              // Zhler
  this.denominator = b;            // Nenner
            
  // Methoden
  // ------------------------------
    
  // Unre Operationen:
  
  this.sign = sign                 // Vorzeichen    
  this.negated = negated           // Vorzeichenwechsel
  this.absolute = absolute         // Absolutbetrag
  this.reciprocal = reciprocal     // Kehrwert
  this.cancelled = cancelled       // Krzen
  this.floor = floor               // grter ganzer Wert kleinergleich
  this.ceiling = ceiling           // kleinster ganzer Wert grergleich
  this.asMixed = asMixed           // Darstellung als Gemischtbruch
  this.asFloat = asFloat           // Gleitpunktnherung 
    
  // Binre Operationen (innerhalb der Klasse "fraction"):

  this.times = times               // Multiplizieren
  this.extended = extended         // Erweitern  
  this.divide = divide             // Dividieren
  this.plus = plus                 // Addieren
  this.minus = minus               // Subtrahieren
    
  // Binre Operationen (booleschwertig):
    
  this.equal = equal               // Test auf Gleichheit
  this.less = less                 // Test auf Kleiner
  this.lessEqual = lessEqual       // Test auf Kleinergleich
  this.greater = greater           // Test auf Grer
  this.greaterEqual = greaterEqual // Test auf Grergleich
}


function GGT(a,b)
// Hilfsfunktion (keine Methode):
// Berechnet den grten gemeinsamer Teiler von a, b
// nach dem Euklidischen Algorithmus (iterativ)
{
  a = Math.abs(a);
  b = Math.abs(b);
  var rest;
  
  while ( b>0 )
  {
    rest = a % b;    // Division mit Rest
    a = b;           // Divisor wird Dividend
    b = rest;        // Rest wird Divisor
  }
  return a           // letzter nicht-verschwindender Rest
}


function Quotient(a,b)
// Berechnet das Ganze der Division a:b
{
  return (a - ( a % b )) / b
}


function sign()
// Methode: berechnet das Vorzeichen
// Rckgabe: -1, 0, 1, Undefined
{
  var a = parseInt(this.numerator);
  var b = parseInt(this.denominator);

  if ( b>0 )
    {
      if ( a>0 ) return 1
      else if ( a<0 ) return -1
      else return 0
    }
  else if ( b<0 )
    {
      if ( a>0 ) return -1
      else if ( a<0 ) return 1
      else return 0
    }
  else return "Undefined"  
}


function negated()
// Methode: wechselt das Vorzeichen
// via "call by reference"
{
  var a = parseInt(this.numerator);
  var b = parseInt(this.denominator);
  var vorzeichen = this.sign();
  
  if ( vorzeichen>0 )
    {
      this.numerator = -Math.abs(a);
      this.denominator = Math.abs(b);
    }
  else if ( vorzeichen<0 )
    {
      this.numerator = Math.abs(a);
      this.denominator = Math.abs(b);
    }  
}


function absolute()
// Methode: berechnet den Absolutbetrag
// via "call by reference"
{
  this.numerator = Math.abs(this.numerator);
  this.denominator = Math.abs(this.denominator)
}


function reciprocal()
// Methode: berechnet den Kehrwert
// via "call by reference"
{
  var a = parseInt(this.numerator);
  var b = parseInt(this.denominator);
  this.numerator = b;
  this.denominator = a
}


function cancelled()
// Methode: krzt den Bruch
// via "call by reference"
{
  var vorzeichen = this.sign();
  var a = Math.abs(this.numerator);
  var b = Math.abs(this.denominator);
  var t = GGT(a,b); // Teiler
  
  if ( b>0 ) // dann auch t>0 (!)
    {
      // Vorzeichen im Zhler erzwingen:
      this.numerator = vorzeichen*(a/t);
      this.denominator = b/t;
    };
  return vorzeichen
}


function floor()
// Methode: rundet ganzzahlig ab
// Rckgabe: Rundungswert
{
  var rval; // Rckgabewert
  var a = parseInt(this.numerator);
  var b = parseInt(this.denominator);
  
  if ( b!=0 )
    rval = Quotient(a,b)
  else
    {
      if ( a>0 ) 
        rval = "+Infinity"
      else if ( a<0 )
        rval = "-Infinity"
      else
        rval = "Undefined"       
    }
  return rval  
}


function ceiling()
// Methode: rundet ganzzahlig auf
// Rckgabe: Rundungswert
{ 
  if ( this.denominator != 0 )
    {
      if ( this.numerator % this.denominator != 0 )
        return this.floor() + 1
      else
        return this.floor()
    }
  else
    return this.floor()
}


function asMixed()
// Methode: wandelt Bruch in Gemischtbruch um
// Rckgabe: Ganzteil (vorzeichenbehaftet)
// ferner: "call by reference" (-> Restebruch)
{
  var ganz = 0; // Rckgabewert
  
  var a = parseInt(this.numerator);
  var b = parseInt(this.denominator);
  var vorzeichen = this.sign();

  if ( b==0 )
    {
      if ( a>0 )
        ganz = "+Infinity"
      else if ( a<0 )
        ganz = "-Infinity"
      else
        ganz = "Undefined"
    }
  else if ( Math.abs(a)>=Math.abs(b) )
    {
      a = Math.abs(a); b = Math.abs(b);
      ganz = vorzeichen * Quotient(a,b);
      this.numerator = a-b*Math.abs(ganz);
      this.denominator = b
    }
  return ganz  
}


function asFloat()
// Methode: wandelt Bruch in Gleitpunktzahl um
// Rckgabe: Gleitpunktwert (Nherung)
{
  var a = parseInt(this.numerator);
  var b = parseInt(this.denominator);
  var rval = parseFloat(a/b); // Rckgabewert
  if ( b==0 )
    {
      if ( a>0 )
        rval = "+Infinity"
      else if ( a<0 )
        rval = "-Infinity"
      else
        rval = "Undefined"
    }
  else if ( isNaN(rval) )
      rval = "Undefined";  
  return rval      
}


function times( bruch )
// Methode: Multiplikation mit einem Bruch
// Rckgabe: gekrztes Produkt
{
  var produkt = new Fraction(0,0);
  produkt.numerator = this.numerator * bruch.numerator;
  produkt.denominator = this.denominator * bruch.denominator;
  produkt.cancelled();
  return produkt
}


function extended( faktor )
// Methode: Erweitern mit einem Faktor
// via "call by reference"
{
  this.numerator = this.numerator * faktor;
  this.denominator = this.denominator * faktor
}


function divide( bruch )
// Methode: Dividieren durch einen Bruch
// Rckgabe: gekrzter Quotient
{
  var quotient = new Fraction(0,0);
  bruch.reciprocal();
  quotient = this.times( bruch ); 
  return quotient
}


function plus( bruch )
// Methode: Addition eines Bruchs
// Rckgabe: gekrzte Summe
{
  var summe = new Fraction(0,0);
  summe.numerator = this.numerator * bruch.denominator + bruch.numerator * this.denominator;
  summe.denominator = this.denominator * bruch.denominator;
  summe.cancelled();
  return summe
}


function minus( bruch )
// Methode: Subtraktion eines Bruchs
// Rckgabe: gekrzte Differenz
{
  var differenz = new Fraction(0,0);
  bruch.negated()
  differenz = this.plus( bruch );
  return differenz
}


function equal( bruch )
// Methode: Test auf Gleichheit zu bruch
// Rckgabe: boolescher Wert
{
  return ( this.numerator * bruch.denominator == this.denominator * bruch.numerator )
}


function less( bruch )
// Methode: Test auf kleiner als bruch
// Rckgabe: boolescher Wert
{
  var rval = false; // Rckgabewert

  var a1 = Math.abs(parseInt( this.numerator ));
  var b1 = Math.abs(parseInt( this.denominator ));
  var a2 = Math.abs(parseInt( bruch.numerator ));
  var b2 = Math.abs(parseInt( bruch.denominator ));
  
  if ( b1>0 && b2>0 )
    {
      if ( this.sign()<0 ) a1 = -a1;
      if ( bruch.sign()<0 ) a2 = -a2;
      rval = ( a1 * b2 < a2 * b1  )
    }
  return rval  
}


function lessEqual( bruch )
// Methode: Test auf kleiner oder gleich bruch
// Rckgabe: boolescher Wert
{
  return ( this.less( bruch ) || this.equal( bruch ) )
}


function greater( bruch )
// Methode: Test auf grer als bruch
// Rckgabe: boolescher Wert
{
  return ( !this.less( bruch ) && !this.equal( bruch )  ) 
}


function greaterEqual( bruch )
// Methode: Test auf grer oder gleich bruch
// Rckgabe: boolescher Wert
{
  return !this.less( bruch )
}


///////////////////////////////////////////////////////////////////
//-----------------------------------------------------------------